<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports Services
 *
 * @package    ADSL
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Service_Plugin_Adsl extends Model_Service_Plugin {
	protected $_table_name = 'service__adsl';
	protected $_updated_column = FALSE;

	// Relationships
	protected $_belongs_to = array(
		'service'=>array(),
	);

	protected $_has_one = array(
		'adsl_plan'=>array('model'=>'ADSL_Plan','far_key'=>'provided_adsl_plan_id','foreign_key'=>'id'),
	);

	protected $_display_filters = array(
		'service_connect_date'=>array(
			array('Config::date',array(':value')),
		),
		'service_contract_date'=>array(
			array('Config::date',array(':value')),
		),
	);

	// Required abstract functions
	public function admin_update() {
		return View::factory('service/admin/adsl/update')
			->set('mediapath',Route::get('default/media'))
			->set('so',$this);
	}

	/**
	 * When does this service expire
	 */
	public function expire() {
		// @todo This should work out if the invoices are currently due, then the expire is the invoice date, otherwise the next invoice date.
		return NULL;
	}

	public function name() {
		return $this->service_number;
	}

	public function product() {
		if ($this->provided_adsl_plan_id)
			return $this->adsl_plan;
		else
			return $this->service->product->plugin();
	}

	public function service_view() {
		return View::factory('service/user/plugin/adsl/view')
			->set('o',$this);
	}

	public function username_value() {
		return $this->service_username;
	}

	public function password_value() {
		return $this->service_password;
	}

	/**
	 * Return the IP Address for the service
	 */
	public function ipaddress() {
		return $this->ipaddress ? $this->ipaddress : _('Dynamic');
	}

	public function contract_date_start() {
		return Config::date($this->service_contract_date);
	}

	public function contract_date_end($format=FALSE) {
		$x = strtotime(sprintf('+%s months',$this->contract_term),$this->service_contract_date);

		return $format ? Config::date($x) : $x;
	}

	public function hasOffpeak() {
		return ((is_null($this->product()->base_down_offpeak) OR $this->product()->base_down_offpeak) AND (is_null($this->product()->base_up_offpeak) OR $this->product()->base_up_offpeak)) ? TRUE : FALSE;
	}

	/**
	 * This function will return the months that have traffic data.
	 * This array can be used in a select list to display the traffic for that month
	 */
	public function get_traffic_months() {
		$keys = $months = array();

		foreach ($this->get_traffic_data_monthly() as $metric => $data)
			$keys = array_unique(array_merge($keys,array_keys($data)));

		foreach ($keys as $v)
			$months[$v] = $v;

		arsort($months);

		return $months;
	}

	/**
	 * Calculate the total traffic used in a month
	 */
	private function get_traffic_data_month($period=NULL,$cache=0) {
		$result = array();

		foreach ($this->get_traffic_data_daily($period,TRUE,$cache) as $tdata)
			foreach ($tdata as $k => $v)
				if (isset($result[$k]))
					$result[$k] += $v;
				else
					$result[$k] = $v;

		return $result;
	}

	/**
	 * Return an array of the data used in a month by day
	 */
	public function get_traffic_data_daily($period=NULL,$bydate=FALSE,$cache=0) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');

		$t = ORM::factory('Service_Plugin_Adsl_Traffic')
			->where('service','=',$this->service_username)
			->and_where('date','>=',date('Y-m-d',mktime(0,0,0,date('m',$period),1,date('Y',$period))))
			->and_where('date','<=',date('Y-m-d',strtotime('last day of '.date('M Y',$period))))
			->cached($cache);

		foreach ($t->find_all() as $to) {
			$day = date('d',strtotime($to->date));

			if ($bydate)
				$result[$day] = $to->traffic($this->service->product->plugin());
			else
				foreach ($to->traffic($this->service->product->plugin()) as $k => $v)
					$result[$k][$day] = $v;
		}

		return $result;
	}

	/**
	 * Return an array of the data used in a year by month
	 */
	public function get_traffic_data_monthly($period=NULL,$bydate=FALSE) {
		$result = array();

		if (is_null($period))
			$period = strtotime('yesterday');

		$t = ORM::factory('Service_Plugin_Adsl_Traffic')
			->select(
				array('date_format(date,\'%y-%m\')','month'),
				array('sum(up_peak)','up_peak'),
				array('sum(up_offpeak)','up_offpeak'),
				array('sum(down_peak)','down_peak'),
				array('sum(down_offpeak)','down_offpeak')
			)
			->where('service','=',$this->service_username)
			->and_where('date','>=',date('Y-m-d',mktime(0,0,0,date('m',$period),1,date('Y',$period)-1)))
			->and_where('date','<=',date('Y-m-d',strtotime('last day of '.date('M Y',$period))))
			->group_by('date_format(date,\'%Y-%m\')');

		foreach ($t->find_all() as $to)
			if ($bydate)
				$result[$to->month] = $to->traffic($this->service->product->plugin());
			else
				foreach ($to->traffic($this->service->product->plugin()) as $k => $v)
					$result[$k][$to->month] = $v;

		return $result;
	}

	public function traffic_month($month,$string=TRUE,$cache=0) {
		return $string ? implode('/',$this->get_traffic_data_month($month,$cache)) :
			$this->get_traffic_data_month($month,$cache);
	}

	public function traffic_lastmonth($string=TRUE) {
		// We need it to be last month as of yesterday
		return $this->traffic_month(strtotime('last month')-86400,$string);
	}

	public function traffic_thismonth($string=TRUE) {
		return $this->traffic_month(strtotime('yesterday'),$string);
	}

	public function traffic_lastmonth_exceed($all=FALSE,$date=NULL) {
		$result = array();

		if (is_null($date))
			$date = strtotime('last month')-86400;

		foreach ($this->traffic_month($date,FALSE,0) as $k => $v) {
			// We shouldnt need to eval for nulls, since the traffic calc does that
			if ($all OR ($v > $this->service->product->plugin()->$k)) {
				$result[$k]['allowance'] = $this->service->product->plugin()->$k;
				$result[$k]['used'] = $v;
				$result[$k]['shaped'] = (! empty($this->service->product->plugin()->extra_shaped) AND $this->service->product->plugin()->extra_shaped AND $v > $this->service->product->plugin()->$k) ? TRUE : FALSE;
				$result[$k]['excess'] = (! empty($this->service->product->plugin()->extra_charged) AND $this->service->product->plugin()->extra_charged AND $v > $this->service->product->plugin()->$k) ? $v-$this->service->product->plugin()->$k : 0;
				$result[$k]['rate'] = $this->service->product->plugin()->{ADSL::map($k)};
				$result[$k]['charge'] = ceil(($result[$k]['excess'])/1000)*$result[$k]['rate'];
			}
		}

		return $result;
	}

	public function template_variables($array) {
		$friendly = array(
			'base_down_peak'=>'Peak',
			'base_down_offpeak'=>'OffPeak',
			'cumulative_base_down_peak'=>'Total Peak',
			'cumulative_base_down_offpeak'=>'Total OffPeak',
		);

		$result = array();
		if ($this->service->product->prod_plugin_file != 'ADSL')
			throw new Kohana_Exception('Huh? How did this get called, for a non ADSL product (:ppf)',array(':ppf'=>$this->service_id));

		$allowance = $this->service->product->plugin()->allowance(FALSE);

		$period = strtotime('yesterday');
		$traffic_data = $this->get_traffic_data_daily($period,FALSE);
		$traffic = $this->get_traffic_data_month($period);

		$traffic_type = $this->get_traffic_data_daily($period,TRUE);
		$day = count($traffic_type) ? max(array_keys($traffic_type)) : 1;
		$date = mktime(0,0,0,date('n',$period),$day,date('Y',$period));
		$daysleft = date('d',strtotime('last day of',$date))-$day;

		$google = GoogleChart::factory('Legacy')
			->type('vertical_bar')
			->title(sprintf('DSL traffic usage as at %s',Config::date($date)));

		foreach ($traffic_data as $k => $details)
			$google->sdata(array('yl'=>($x=isset($friendly[$k]) ? $friendly[$k] : $k)),array($x=>$traffic_data[$k]));

		// Work out comulative numbers
		foreach ($traffic_data as $k => $details)
			$google->sdata(array('yr'=>($x=isset($friendly['cumulative'.$k]) ? $friendly['cumulative'.$k] : 'cumulative'.$k)),array($x=>$this->cumulative($traffic_data[$k])));

		foreach ($array as $item) {
			switch ($item) {
				case 'MONTH_GRAPH': $value = (string)$google; break;
				case 'MONTH_TABLE': $value = $google->table(FALSE,array(
					'table'=>'style="border: 1px solid #bebcb7; padding: 5px 5px; background: none repeat scroll 0% 0% #f8f7f5; font-size: 70%;"',
					)); break;

				case 'OFFPEAK_ALLOWANCE': $value = isset($allowance['base_down_offpeak']) ? $allowance['base_down_offpeak'].' MB' : '-'; break;
				case 'OFFPEAK_USAGE': $value = isset($traffic['base_down_offpeak']) ? $traffic['base_down_offpeak'].' MB' : '-'; break;

				case 'PEAK_ALLOWANCE': $value = isset($allowance['base_down_peak']) ? $allowance['base_down_peak'].' MB' : '-'; break;
				case 'PEAK_USAGE': $value = isset($traffic['base_down_peak']) ? $traffic['base_down_peak'].' MB' : '-'; break;

				case 'OFFPEAK_AVERAGE': $value = isset($traffic['base_down_offpeak']) ? round($traffic['base_down_offpeak']/$day,2).' MB' : '-'; break;
				case 'OFFPEAK_AVERAGE_REMAIN': $value = ((isset($traffic['base_down_offpeak']) AND ($allowance['base_down_offpeak'] > $traffic['base_down_offpeak']) AND $daysleft) ? round(($allowance['base_down_offpeak']-$traffic['base_down_offpeak'])/$daysleft,2).' MB' : '-'); break;

				case 'PEAK_AVERAGE': $value = isset($traffic['base_down_peak']) ? round($traffic['base_down_peak']/$day,2).' MB' : '-'; break;
				case 'PEAK_AVERAGE_REMAIN': $value = ((isset($traffic['base_down_peak']) AND ($allowance['base_down_peak'] > $traffic['base_down_peak']) AND $daysleft) ? round(($allowance['base_down_peak']-$traffic['base_down_peak'])/$daysleft,2).' MB' : '-'); break;

				case 'SERVICE_NUMBER': $value = $this->service_number; break;
				case 'USAGE_DATE': $value = Config::date($date); break;
				case 'USER_NAME': $value = $this->service->account->name(); break;
				default:
					$value = '';
			}

			$result[$item] = $value;
		}

		return $result;
	}

	/**
	 * This function will take an array of numbers and change it into a cumulative array
	 */
	public function cumulative($array) {
		$result = array();
		$s = 0;

		foreach ($array as $k => $v)
			$result[$k] = ($s += $v);

		return $result;
	}

	/**
	 * Determine if we alert traffic
	 *
	 * We alert traffic if:
	 * + 80% of usage every 3 days
	 * + average daily usage > allowance every 5 days
	 * + last day of the period
	 */
	public function report_traffic() {
		if ($this->service->product->prod_plugin_file != 'ADSL')
			throw new Kohana_Exception('Huh? How did this get called, for a non ADSL product (:ppf)',array(':ppf'=>$this->service_id));

		$allowance = $this->service->product->plugin()->allowance(FALSE);

		$period = strtotime('yesterday');
		$traffic_data = $this->get_traffic_data_daily($period,FALSE);
		$traffic = $this->get_traffic_data_month($period);

		$traffic_type = $this->get_traffic_data_daily($period,TRUE);
		// @todo If no data comes in, then this can be stuck reporting traffic for an old date.
		$day = count($traffic_type) ? max(array_keys($traffic_type)) : 1;
		$lastday = date('d',strtotime('last day of',$period));

		foreach ($traffic as $k => $v) {
			// If we are the last day of the period
			if ($day == $lastday AND $v)
				return TRUE;

			// If we are at 80% usage
			if ($v/($allowance[$k] > 0 ? $allowance[$k] : 1) >= .8 AND $day%3 == 0)
				return TRUE;

			// If our average is greater than our allowance
			if ($day%5 == 0 AND ($v/$day > $allowance[$k]/$day))
				return TRUE;
		}

		// If we get here, then we dont need to report usage.
		return FALSE;
	}

	/**
	 * Get specific service details for use in other modules
	 * For Example: Invoice
	 *
	 * @todo Make the rendered items configurable
	 * @todo Change this method name, now that it is public
	 */
	public function _details($type) {
		switch ($type) {
			case 'invoice_detail_items':
				return array(
					_('Service Address')=>$this->service_address ? $this->display('service_address') : '>NotSet<',
					_('Contract Until')=>$this->contract_date_end(TRUE),
				);
				break;
			default:
				return parent::$_details($type);
		}
	}

	/**
	 * Render a google chart of traffic
	 */
	public function graph_traffic($month=NULL) {
		$google = GoogleChart::factory('Legacy')
			->type('vertical_bar');

		// If we came in via a post to show a particular month, then show that, otherwise show the yearly result
		if (! is_null($month) AND trim($month)) {
			$google->title(sprintf('DSL traffic usage for %s',$_POST['month']));
			$traffic_data = $this->get_traffic_data_daily(strtotime($_POST['month'].'-01'));

			foreach ($traffic_data as $k => $details)
				$google->sdata(array('yl'=>($x=isset($friendly[$k]) ? $friendly[$k] : $k)),array($x=>$traffic_data[$k]));

			foreach ($traffic_data as $k => $details)
				$google->sdata(array('yr'=>($x=isset($friendly['cumulative'.$k]) ? $friendly['cumulative'.$k] : 'cumulative'.$k)),array($x=>$this->cumulative($traffic_data[$k])));

		} else {
			// @todo Change the date to the last record date
			$google->title(sprintf('Monthly DSL traffic usage as at %s',Config::date(strtotime('yesterday'))));
			$traffic_data = $this->get_traffic_data_monthly();

			foreach ($traffic_data as $k => $details)
				$google->sdata(array('yl'=>($x=isset($friendly[$k]) ? $friendly[$k] : $k)),array($x=>$traffic_data[$k]));
		}

		return (string)$google;
	}

	public function table_traffic($month=NULL) {
		return View::factory('service/user/plugin/adsl/table_traffic')
			->set('traffic',$this->traffic_month((! is_null($month) AND trim($month)) ? strtotime($month.'-01') : NULL,FALSE));
	}

	/**
	 * Search for services matching a term
	 */
	public function list_autocomplete($term,$index,$value,array $label,array $limit=array(),array $options=NULL) {
		// We only show invoice numbers.
		if (! is_numeric($term))
			return array();

		$ao = Auth::instance()->get_user();

		$options['key'] = 'id';
		$options['object'] = DB::select($this->_table_name.'.id',$this->_table_name.'.service_number')
			->from($this->_table_name)
			->join('service')
			->on('service.id','=',$this->_table_name.'.service_id')
			->where('service.account_id','IN',$ao->RTM->customers($ao->RTM))
			->and_where($this->_table_name.'.service_number','like','%'.$term.'%');

		return parent::list_autocomplete($term,$index,$value,$label,$limit,$options);
	}
}
?>
