<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class supports a product cart
 *
 * @package    Cart
 * @category   Models
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Model_Cart extends ORM_OSB {
	protected $_belongs_to = array(
		'product'=>array(),
	);

	// Cart doesnt use the update column
	protected $_updated_column = FALSE;

	protected $_serialize_column = array(
		'module_data',
	);

	/**
	 * Filters used to format the display of values into friendlier values
	 */
	protected $_display_filters = array(
		'recurr_schedule'=>array(
			array('StaticList_RecurSchedule::get',array(':value')),
		),
	);

	private $mo;

	public function __construct($id = NULL) {
		// Load our Model
		parent::__construct($id);

		// Autoload our Sub Items
		if ($this->loaded())
			$this->_load_sub_items();

		return $this;
	}

	private function _load_sub_items() {
		$this->mo = ORM::factory('Module',$this->module_id)->instance($this->module_item);

		if (! $this->mo->loaded())
			throw new Kohana_Exception('Item :item not loaded?',array(':item'=>$this->module_item));
	}

	public function checkout() {
		if (! method_exists($this->mo,'checkout'))
			throw new Kohana_Exception('Module :module doesnt implement checkout?',array(':module'=>get_class($this->mo)));

		return $this->mo->checkout();
	}

	public function item() {
		if (! method_exists($this->mo,'cart_item'))
			throw new Kohana_Exception('Module :module doesnt implement cart_item?',array(':module'=>get_class($this->mo)));

		return $this->mo->cart_item();
	}

	public function mo() {
		return $this->mo;
	}

	public function motype() {
		return strtolower(preg_replace('/^Model_/','',get_class($this->mo)));
	}
}
?>
